

/*
 Dieses Script wurde geschrieben von Mitja Stachowiak (www.mitjastachowiak.de)
 Es benoetigt ausserdem frameCommunicator.js.
 
 Dokumentation unter http://www.mitjastachowiak.de/code/js/dynamicIFrame
 
*/





function getTickCount () {
// returns the milliseconds since beginning of the current year
 var T = new Date();
 return T.getMilliseconds() + T.getSeconds() * 1000 + T.getMinutes() * 60000 + T.getHours() * 3600000 + T.getDay() * 86400000;
}
function createOverscrollListener (doc, wait, callback) {
 var o = new Object();
 o.doc = doc;
 o.lastScroll = 0;
 o.lastTry = 0;
 o.lastInnerWidth = window.innerWidth;
 o.wait = wait / 2;
 o.callback = callback;
 o.scrollFkt = function () {
  o.lastScroll = getTickCount();
 }
 o.scrollEventFkt = function (ev) {
  if (getTickCount() < o.lastScroll + o.wait) { o.lastTry = 0; return; }
  if (o.lastTry == 0) o.lastTry = getTickCount();
  if (getTickCount() < o.lastTry + o.wait) { o.lastInnerWidth = window.innerWidth; return; }
  if (o.lastInnerWidth != window.innerWidth) { o.lastInnerWidth = window.innerWidth; return; }
  o.lastTry = 0;
  if (!ev) ev = window.event;
  var dir = ev.wheelDelta || -ev.detail;
  if (!callback) return;
  if (dir) { o.callback(dir < 0); return; }
  if (window.pageYOffset > 0) { o.callback(true); return; }
  if (ev.keyCode) { o.callback(ev.keyCode == 40); return; }
 }
 o.free = function () {
  o.doc.removeEventListener('scroll', o.scrollFkt, false);
  o.doc.removeEventListener('DOMMouseScroll', o.scrollEventFkt, false);
  o.doc.removeEventListener('mousewheel', o.scrollEventFkt, false);
  o.doc.removeEventListener('keypress', o.scrollEventFkt, false);
  o.callback = null;
  o.doc = null;
  o = null;
 }
 o.doc.addEventListener('scroll', o.scrollFkt, false);
 o.doc.addEventListener('DOMMouseScroll', o.scrollEventFkt, false);
 o.doc.addEventListener('mousewheel', o.scrollEventFkt, false);
 o.doc.addEventListener('keypress', o.scrollEventFkt, false);
 return o;
}


/*
 Dies sind zwei standard-Effekte. Diese koennen Sie als Vorlage fuer eigene Effekte verwenden
 
 Parameter:
  el : iframe-Element
  s : Wert zwischen 0 und 1; Ausnahme -1 (0 = Effekt beginnt, 0.5 = Effekt ist bei der Haelfte, 1 = Effekt ist fertig, -1 = saemtliche von diesem Effekt benutzten css-Eigenschaften zuruecksetzen)
  blendIn : Wenn true, soll das iframe eingeblendet werden, ansonsten ausgeblendet
*/

function EffectBlend (el, s, blendIn) {
 if (s < 0) { el.style.opacity = 1; return; }
 if (!blendIn) s = 1-s;
 el.style.opacity = s;
}

function EffectBlendAndSlide (el, s, blendIn) {
 if (s < 0) { el.style.marginTop = '0px'; el.style.opacity = 1; return; }
 if (!blendIn) s = 1-s;
 el.style.opacity = s;
 el.style.marginTop = -100 * (1-s) * (1-s) + 'px';
}

function EffectAppendAndSlide (el1, el2, s, dir) {
 if (s < 0) {
  el1.style.marginTop = '0px';
  el2.style.marginTop = '0px';
  el1.parentElement.style.overflow = 'visible';
  return;
 }
 el1.parentElement.style.overflow = 'hidden';
 if (dir) {
  el1.style.marginTop = -el1.offsetHeight * (1-s);
  el2.style.marginTop = el1.offsetHeight * (s);
 } else {
  el2.style.marginTop = -el2.offsetHeight * (s);
  el1.style.marginTop = el2.offsetHeight * (1-s);
 }
}




/*
 Dieses Objekt darf nur einmal in einem HTML-Dokument erstellt werden und erlaubt, das Laden von Seiten in einem iframe mit Effekt zu versehen.
 
 Parameter:
  FrameBox : ein DIV-Element, das genau 2 iframes enthaellt. Jedes IFrame benoetigt ein eindeutriges name-Attribut. Nur so kann die Kompatibilitaet mit Links ohne JavaScript gewaehrleistet werden (<A href="frameSeite.html" target="frameName">).
 
 Eigenschaften und Funktionen:
  whitelist : Array aus RegExp-Objekten. Alle Seiten, die im Frame geladen werden sollen, muessen entweder von der gleichen Domain sein, oder auf die Whitelist passen
  addURLToWhitelist : Wandelt die uebergebene URL in ein RegExp-Objekt um und fuegt sie der whitelist hinzu. In der URL sind *-Zeichen erlaubt. Innerhalb des Hostname-Bereiches haben diese jedoch spezielle Aufgaben und es kann nicht jede beliebige Zeichenfolge fuer ein * stehen.
  differentQueryDifferentSites : Boolean - wenn true, dann werden Menuelinks nur hervorgehoen, wenn auch das QUerystring des Menuelinks und der Seite im Frame gleich ist.
  whitelist : Array aus Strigs (Darf Platzhalter (*) enthalten). Nur Seiten, die in dieses Profil passen, duerfen im Frame geladen werden. Seiten von der eigenen Domain duerfen immer geladen werden.
  maxWaitForSite : diese Zeit (ms) wird maximal gewartet, bis die neue Seite eingeblendet wird. Ist die Seite bis dahin nicht vollstaendig geladen, kann der Besucher beobachten, wie sie im iframe fertig laedt.
  blendOutTime : Dauer des Ausblendens (ms)
  blendInDelay : Verzoegerung - so viele Millisekunden nach Beginn des Ausblendens beginnt das Einblenden
  blendInTime : Dauer des Einblendens (ms)
  blendInEffect : Uebergeben Sie hier die Effekt-Funktion fuer das Einblenden
  blendOutEffect : Uebergeben Sie hier die Effekt-Funktion fuer das Ausblenden
  function loadSite : Rufen Sie diese Funktion mit einer URL auf, um diese URL mit Effekt in den iframe zu laden (Die Seite wird nur geladen, wenn sie von der gleichen Domain ist oder auf die Whitelist passt)
  function scrollTo : uebergeben werden kann eine y-Koordinate aus Sicht des aktuellen Iframes oder der Name eines Ankers innerhalb des aktuellen Iframes. ScrollTo scrollt dann zu diesem Ziel. Ein zweiter Parameter darf nur vom Objekt selbst uebergeben werden.
  autoFillMenu : Falls die Menu-Links ein Inhaltsverzeichnis darstellen, kann fuer autoFillMenu eine Zahl eingesetzt werden, bis zu der H*-Elemente der Frame-Seite gescant und automatisch im Inhaltsverzeichnis ergaenzt werden. Ein Wert von 3 z.B. bedeutet, dass fuer alle H1, H2 und H3-Elemente automatisch Links im Inhaltsverzeichnis beim Laden der Seite angelegt werden.
*/
function DynamicIFrame (FrameBox) { with (this) {
 // private
 var box = FrameBox;
 var frames = FrameBox.getElementsByTagName('iframe');
 var current = 0;
 var next = 1;
 var sites = new Array();
 var communicators = new Array();
 var aTarget, aTarget2;
 var isAutoFrameHeight = true;
 var loadByQueryString = false;
 var menuLinks = new Array();
 var printLinks = new Array();
 var blendInfo = new Object();
 var scrollInfo = new Object();
 var autoHeightEl = new Array(); // Alle Elemente, deren NoJsHeight-Classname gesetzt/entfernt wird
 var loadWithoutEffect = false;
 var startPage = '';
 var isFirstPage = true;
 var incompatibleBrowser = false;
 var domWasReady = false;
 var selfHostname;
 var currentSite = '';
 var loadingSiteAnchor = ''; // Anker der zu ladenden Seite aus URL, wird gespeichert fuer scroll-Effekt
 var currentSiteAnchors = new Array(); // Array aus allen Ueberschrifen-Ankern der aktuellen Seite (kann nach absOffsetTop der Anker sortiert werden)
 var anchorPos = 0; // index in currentSiteAnchors des gerade aktiven Ankers
 var overscrollListener = null;

 // public
 this.differentQueryDifferentSites = false;
 this.whitelist = new Array();
 this.maxWaitForSite = 1000; // Nach dieser Zeit wird die Seite anzegeigt, auch wenn noch nicht vollst. geladen
 this.blendOutTime = 0;
 this.blendInDelay = 0;
 this.blendInTime = 0;
 this.scrollTime = 500;
 this.blendInEffect = null;
 this.blendOutEffect = null;
 this.browseEffect = EffectAppendAndSlide;
 this.browseDuration = 2000;
 this.scrollFunction = function (s) { return (-2*s+3)*s*s; }
 this.browseScrollFunction = function (s) { return s; }
 this.autoFillMenu = 0; // Bis zu dieser Zahl * werden H*-Elemente, die einen Anker enthalten im Menue ergaenzt
 this.menuGroup = null; // falls autoFillMenu nicht 0 ist, muss hier das hoechste OL-Element des Menues eingetragen werden
 this.autoChangeAnchor = false;
 this.nummerateMenu = false;
 this.onSiteLoaded = null;
 this.onLinkActivate = null;
 this.onMenuChange = null;
 this.searchHeadlinesWithoutAnchor = true;
 
 var _this = this;
 var onEvent = /* private */ function (el, event, fkt, param, behavior) { // provides easy eventListeners. el: Element to set the listener; event: string-name of the event(without "on"); fkt: a Function to call ift the event occours, fkt is called in the scope of _this; param: any parameter, that is given to fkt; behavior: flags - 1=only the param is given to fkt (otherwise there is also given the element and the event), 2=event is not given to the element
  var f;
  if (behavior & 1) f = function (ev) {
    if (!ev) ev = window.event;
 	fkt.call(_this,param);
	if (behavior & 2) { if (ev.preventDefault) ev.preventDefault(); return false; }
   }
  else f = function (ev) {
    if (!ev) ev = window.event;
 	fkt.call(_this,this,ev,param);
    if (behavior & 2) { if (ev.preventDefault) ev.preventDefault(); return false; }
   }
  if (window.addEventListener) el.addEventListener(event, f, false);
  else el.attachEvent('on'+event,f);
  return false;
 }
 
 var getAbsOffsetTop = function (el) { // returns the offset top of the given element seen from the document top (el CAN be in the current IFRAME)
  var pos = el.offsetTop;
  while (el.offsetParent) {
   if (el.offsetParent == el) break;
   el = el.offsetParent;
   pos += el.offsetTop;
  }
  try {
   if (el == sites[current].document.body || el == sites[current].document.documentElement) pos += getAbsOffsetTop(frames[current]);
  } catch (e) { }
  return pos;
 }
 
 var getAnchorFromURL = /* private */ function (url) {
  var i = url.lastIndexOf('#');
  if (i < 0) return '';
  var a = url.substring(i+1, url.length);
  if ((/[^(A-Za-z0-9_)]/).test(a)) return '';
  return a;
 }
 
 var getSiteFromURL = /* private */ function (url) {
  var i = url.lastIndexOf('#');
  var j = url.indexOf('?');
  if (!differentQueryDifferentSites && j >= 0 && j < j) i = j;
  if (i < 0) i = url.length;
  return url.substring(0,i);
 }

 var create = /* private */ function () {
  if ((!window.addEventListener && !window.attachEvent) || !document.body || !document.getElementsByTagName) { incompatibleBrowser = true; return; }
  if (!window.fapi) { incompatibleBrowser = true; throw 'DynamicIFrame benoetigt frameCommunicator.js!'; }
  if (window.jsw && jsw.units.switchableDesign) jsw.units.switchableDesign.unit.requireDesignFile('dynamicIFrame', box);
  selfHostname = getHostnameAndProtocol(self.location.href);
  // Window-Objekt von Frame-Seiten preparieren
  aTarget = frames[0].getAttribute('name', 0);
  aTarget2 = frames[1].getAttribute('name', 0);
  startPage = frames[0].src;
  sites[0] = frames[0].contentWindow;
  sites[1] = frames[1].contentWindow;
  communicators[0] = new FrameCommunicator(frames[0]);
  communicators[1] = new FrameCommunicator(frames[1]);
  communicators[0].autoHeight = true;
  communicators[1].autoHeight = true;
  communicators[0].onContentResize = function (w, h) { siteResize(0, h); };
  communicators[1].onContentResize = function (w, h) { siteResize(1, h); };
  communicators[0].onURLChanged = setQueryString;
  communicators[1].onURLChanged = setQueryString;
  communicators[0].onload = manipulateCurrentSite;
  communicators[1].onload = manipulateCurrentSite;
  if (document.addEventListener) document.addEventListener("DOMContentLoaded", domReady, false);
  else {
   var oldReadyStateChange = document.onreadystatechange;
   document.onreadystatechange = function () {
     try { oldReadyStateChange(); } catch (e) { }
     if(document.readyState == "interactive" || document.readyState == "complete") domReady();
    }
  }
  // QueryString untersuchen
  if (window.addEventListener) window.addEventListener('popstate', evalQueryString, false); else window.attachEvent('onpopstate', function () {evalQueryString(window.event)});
 }
 var domReady = /* private */ function () {
  if (domWasReady) return;
  domWasReady = true;
  // Links auf JavaScript umstellen
  for (var i = 0; i < document.getElementsByTagName('a').length; i++) {
   var el = document.getElementsByTagName('a')[i];
   if (el.target == aTarget) {
    if (el.className.indexOf('MenuLink') >= 0) { // Menue-Links, die hervorgehoben werden sollen, vorbereiten
     var O = new Object(); // menuLinks koennen auch in manipulateCurrentSite erzeugt werden!
     O['el'] = el;
	 O['href'] = absolutePath(el.href);
     O['anchor'] = getAnchorFromURL(O['href']);
     O['site'] = getSiteFromURL(O['href']);
	 O['active'] = false;
     if (autoFillMenu != 0 && menuGroup != null) {
      var group = el;
      var groupNr = 0;
      while (group != menuGroup && group.parentElement) {
       if (group.tagName.toLowerCase() == 'ol') groupNr++;
       group = group.parentElement;
      }
      O['groupNr'] = groupNr;
     }
     menuLinks.push(O);
    }
	onEvent(el, 'click', loadSite, el.href, 3);
   }
   if (el.href.toLowerCase() == 'javascript:'+aTarget.toLowerCase()+'.print()') printLinks.push(el);
  }
  if (nummerateMenu) buildMenuNumbers();
  activateCurrentLink();
  // AutoHeight-Elemente finden
  for (var i = 0; i < document.getElementsByTagName('div').length; i++) if (document.getElementsByTagName('div')[i].className.indexOf('NoJsHeight') >= 0) autoHeightEl.push(document.getElementsByTagName('div')[i])
  if (!isAutoFrameHeight) {
   isAutoFrameHeight = true;
   autoFrameHeight(false);
  }
  // Scroll-Event setzen
  window.addEventListener('scroll', sitescroll, false);
  // Seite aus Querystring laden
  if (!evalQueryString()) manipulateCurrentSite(0);
 }
 
 var evalQueryString = /* private */ function (effect) {
  // Wird in domready und beim klicken auf den Vorwaerts/Rueckwaerts-Button im Browser aufgerufen
  var i = self.location.href.indexOf('?');
  if (i < 0) return false;
  var newSite = absolutePath(self.location.href.substring(i+1, self.location.href.length));
  if (isNotInWhitelist(newSite)) return false;
  loadSite(newSite, (effect)?0:1);
  return true;
 }

 this.addURLToWhitelist = /* public */ function (url) {
  var rep = new RegExp('([^\\w&^\\*])', 'g');
  var host = getHostnameAndProtocol(url);
  var i = host.length;
  host = host.replace(rep, '\\$1');
  var path = url.substring(i, url.length).replace(rep, '\\$1');
  if (path == '' && host.length > 0 && host.charAt(host.length-1) == '*') path = '*';
  while (host.length > 0 && host.charAt(host.length-1) == '*') host = host.substring(0, host.length-1);
  if (host == '') throw 'URL has wrong syntax';
  rep = new RegExp('\\*', 'g');
  path = path.replace(rep, '[a|^a]*'); // innerhalb des path-Bereiches koennen die * fuer beliebige Inhalte stehen
  // innerhalb des hostname-Bereiches haben die * spezielle Aufgaben
  var a = host.split('\\\:\\\/\\\/');
  if (a.length == 1) {
   a = a[0].split('*');
   if (a.length == 2) {
    host = a[0] + '(\\w)*\\:\\/\\/((\\w)+\\.)*' + a[1];
   } else throw 'URL has wrong syntax';
  } else if (a.length == 2) {
   rep = new RegExp('(\\*)', 'g');
   a[0] = a[0].replace(rep, '(\\w)*');
   a[1] = a[1].replace(rep, '((\\w)+\\.)*');
   host = a[0] + '\\:\\/\\/' + a[1];
  } else throw 'URL has wrong syntax';
  whitelist.push(new RegExp('\^'+host+path));
 }
 
 var isNotInWhitelist = /* private */ function (src) {
  if (getHostnameAndProtocol(src) == selfHostname) return false;
  for (var i = 0; i < whitelist.length; i++) if (whitelist[i].test(src)) return false;
  return true;
 }
 
 var buildMenuNumbers = /* private */ function () {
  if (!nummerateMenu) return;
  var a = new Array();
  for (var i = 0; i < menuLinks.length; i++) {
   while (a.length <= menuLinks[i]['groupNr']) a.push(0);
   while (a.length > menuLinks[i]['groupNr']+1) a.pop();
   a[menuLinks[i]['groupNr']]++;
   if (!menuLinks[i]['numberEl']) {
    menuLinks[i]['numberEl'] = document.createElement('span');
    menuLinks[i]['el'].insertBefore(menuLinks[i]['numberEl'], menuLinks[i]['el'].firstChild);
   }
   menuLinks[i]['numberEl'].innerHTML = a.join('.')+'. ';
  }
 }
 
 var activateLink = /* private */ function (link) { // das in link uebergebene MenuLink-Objekt wird aktiviert, ggf. akuell aktive Links werden deaktiviert
  if (window.opera && link) { link['el'].style.visibility = 'hidden'; link['el'].style.visibility = 'visible'; } // Komisches Bug
  if (link && link['active']) return;
  for (var i = 0; i < menuLinks.length; i++) if (menuLinks[i]['active']) {
   menuLinks[i]['active'] = false;
   var c = menuLinks[i]['el'].className.split(' ');
   for (var j = c.length-1; j >= 0; j--) if (c[j] == 'active') c.splice(j, 1);
   menuLinks[i]['el'].className = c.join(' ');
  }
  if (!link) { if (onLinkActivate) onLinkActivate(null); return; }
  link['active'] = true;
  link['el'].className += ' active';
  if (onLinkActivate) onLinkActivate(link['el']);
 }
 
 var activateCurrentLink = /* private */ function () {
  // Seite und Anker ermitteln
  var site = currentSite;
  var anchor = getAnchorFromURL(communicators[current].contentURL);
  // Links mit gleicher Seite suchen
  var a = new Array();
  for (var i = 0; i < menuLinks.length; i++) if (site == menuLinks[i]['site']) a.push(menuLinks[i]);
  // Keinen passenden Link gefunden
  if (a.length == 0) { activateLink(null); return; }
  // Falls aktueller Anker direkt existiert
  for (var i = 0; i < a.length; i++) if (a[i]['anchor'] == anchor) { activateLink(a[i]); return; }
  // Falls keine Ueberschriften-Anker bekannt
  if (currentSiteAnchors.length == 0) {
   activateLink(a[0]);
   return;
  }
  // Falls verknuepfter menuLink schon bekannt
  if (currentSiteAnchors.length != 0 && currentSiteAnchors[anchorPos]['link']) { activateLink(currentSiteAnchors[anchorPos]['link']); return; }
  // Anker-Rangfolge erstellen
  var b = new Array();
  var j = anchorPos-1;
  var h = currentSiteAnchors[anchorPos]['hNr'];
  b[currentSiteAnchors[anchorPos]['hNr']] = currentSiteAnchors[anchorPos];
  for (var i = anchorPos-1; i >= 0; i--) if (currentSiteAnchors[i]['hNr'] < h) {
   h = currentSiteAnchors[i]['hNr'];
   b[h] = currentSiteAnchors[i];
  }
  // naheliegensten Menulink aktivieren
  for (var i = b.length-1; i >= 0; i--) {
   if (!b[i]) continue;
   for (var j = 0; j < a.length; j++) if (a[j]['anchor'] == b[i]['name']) { activateLink(a[j]); return; }
  }
  activateLink(a[0]);
 }
 
 var setQueryString = /* private */ function () { // Diese Funktion haengt an die URL der aktuellen Seite das ?UrlVonFrame an.
  if (loadByQueryString) return;
  var P = communicators[current].contentURL;
  if (P == '') P = communicators[current].contentHostname;
  if (P == '' || P.indexOf('about:blank') == 0) return;
  currentSite = getSiteFromURL(P);
  if (isNotInWhitelist(P)) frames[current].style.visibility = 'hidden';
  else frames[current].style.visibility = 'visible';
  activateCurrentLink();
  var L = self.location.href;
  i = L.indexOf('?');
  if (i < 0) i = L.length;
  var Q = L.substring(i+1, L.length);
  L = L.substring(0, i);
  i = L.indexOf('#');
  if (i > 0) L = L.substring(0, i);
  if (P.toLowerCase() == startPage.toLowerCase()) P = '';
  else {
   var H = getHostnameAndProtocol(communicators[current].contentURL);
   if (H == '') return;
   if (selfHostname == H) P = P.substring(H.length, P.length);
   if (P == Q) return;
   P = '?' + P;
  }
  if (history.pushState && window.location.href != L+P) {
   var stateObj = { foo: "bar" };
   var title = '';
   try { title = sites[current].document.title; } catch (e) {}
   if (isFirstPage) history.replaceState(stateObj, document.title+': '+title, L+P);
   else history.pushState(stateObj, document.title+': '+title, L+P);
  }
 }

 var manipulateCurrentSite = /* private */ function (comm) { // wird von iframe.communicator.onload aufgerufen, sobald der Inhalt des frames geladen hat und dessen Hoehe bekannt ist.
  var frameNr = communicators.indexOf(comm);
  try { sites[frameNr] = frames[frameNr].contentWindow; } catch (e) { }
  if (frameNr != current) return;
  currentSiteAnchors = new Array();
  anchorPos = 0;
  var lastLinkedAnchor = null;
  var linkCreated = false;
  try { // Innerhalb von try-catch: Aenderungen an der Frame-Seite
   sites[current].document.body.style.overflow = 'hidden';
   if (window.jsw) window.jsw.processSite(sites[current].document.body, jswMainWindow);
   var site = currentSite;
   // Ueberschriften ohne Anker ergaenzen
   if (searchHeadlinesWithoutAnchor) for (var i = 1; i <= autoFillMenu; i++) for (var j = sites[current].document.getElementsByTagName('h'+i).length - 1; j >= 0; j--) {
    var h = sites[current].document.getElementsByTagName('h'+i)[j];
    if (h.id == '') continue;
    if (h.getElementsByTagName('a').length == 0 || h.getElementsByTagName('a')[0].parentElement != h || h.getElementsByTagName('a')[0].innerHTML != '') {
     var el = document.createElement('a');
     if (h.firstChild) h.insertBefore(el, h.firstChild); else h.appendChild(el);
    }
   } 
   for (var i = 0; i < sites[current].document.getElementsByTagName('a').length; i++) {
    var el = sites[current].document.getElementsByTagName('a')[i];
    // Links, die auf einen Anker zeigen, mit Effekt versehen, Links, die auf eine Seite Zeigen, durch LoadSite ersetzen (Ausser wenn className WindowLink enthaellt, dann Kompatibilitaet zu JSW herstellen)
    if ((el.target == '_self' || el.target == '') && el.href != '') {
	 var a = el.href.split('#');
     if (a[0].toLowerCase().indexOf('javascript:') != 0 && a[0].toLowerCase().indexOf('callto:') != 0 && a[0].toLowerCase().indexOf('mailto:') != 0 && el.download == '') {
      if (a[0] == sites[current].location.href && a.length > 1) onEvent(el,'click',scrollTo,a[1],3);
      else if ((' '+el.className+' ').indexOf(' WindowLink ') == -1) onEvent(el, 'click', loadSite, el.href, 3);
     }
    }
    // Ueberschriften mit Ankern suchen und im Array currentSiteAnchors speichern
    var aName = (el.id == '')?el.name:el.id;
    if (aName == '') aName = el.parentElement.id;
    if (aName != '' && el.href == '' && (/h\d/i).test(el.parentElement.tagName) && el.parentElement.tagName.length == 2) {
     el.href = '#' + aName;
     var a = new Object(); // a ist "dieser Anker"
     a['el'] = el;
     a['hNr'] = el.parentElement.tagName.charAt(1);
     a['name'] = aName;
     a['content'] = el.parentElement.textContent;
     a['site'] = site;
     a['pos'] = getAbsOffsetTop(el);
     currentSiteAnchors.push(a);
     if (a['hNr'] <= autoFillMenu && menuGroup != null) {
      // Link zu diesem Anker suchen
      for (var j = 0; j < menuLinks.length; j++) if (menuLinks[j]['site'] == site) {
       if (menuLinks[j]['anchor'] == a['name']) {
        lastLinkedAnchor = a;
        a['link'] = menuLinks[j];
        lastLinkedAnchor['linkNr'] = j;
        break;
       }
      }
      // falls kein Link zu diesem Anker gefunden wurde, neuen Link erstellen und in menuLinks einfuegen
      if (!a['link'] && lastLinkedAnchor != null) {
       var link = new Object();
       var thisLinkNr = lastLinkedAnchor['linkNr'] + 1;
	   link['href'] = site + '#' + a['name'];
       link['anchor'] = a['name'];
       link['site'] = site;
       link['groupNr'] = a['hNr']-lastLinkedAnchor['hNr']+lastLinkedAnchor['link']['groupNr'];
	   link['active'] = false;
       link['el'] = document.createElement('a');
       link['el'].textContent = a['content'];
       link['el'].className = 'MenuLink';
       link['el'].target = aTarget;
       link['el'].href = link['href'];
       onEvent(link['el'], 'click', loadSite, link['el'].href, 3);
       menuLinks.splice(thisLinkNr, 0, link);
       a['link'] = link;
       lastLinkedAnchor = a;
       lastLinkedAnchor['linkNr'] = thisLinkNr;
       linkCreated = true;
       if (onMenuChange) onMenuChange();
      }
     }
    }
   }
  } catch (e) {}
  // falls im try-catch-block neue Links erzeugt wurden, menu neu aufbauen
  if (linkCreated) {
   for (var i = 0; i < menuLinks.length; i++) if (menuLinks[i]['el'].parentElement) menuLinks[i]['el'].parentElement.removeChild(menuLinks[i]['el']);
   menuGroup.innerHTML = '';
   var a = new Array(menuGroup);
   for (var i = 0; i < menuLinks.length; i++) {
    while (a.length > menuLinks[i]['groupNr'] + 1) a.pop();
    while (a.length <= menuLinks[i]['groupNr']) {
     var ol = document.createElement('ol');
     a[a.length-1].appendChild(ol);
     a.push(ol);
    }
    var li = document.createElement('li');
    li.appendChild(menuLinks[i]['el']);
    a[a.length-1].appendChild(li);
   }
   if (nummerateMenu) buildMenuNumbers();
  }
  // Menunummern auch in Seite uebernehmen
  if (nummerateMenu) for (var i = 0; i < currentSiteAnchors.length; i++) {
   if (!currentSiteAnchors[i]['link']) continue;
   currentSiteAnchors[i]['el'].innerHTML = currentSiteAnchors[i]['link']['numberEl'].innerHTML;
  }
  // sonstiges
  setQueryString();
  loadByQueryString = false;
  blendInfo['currentSiteReady'] = true;
  if (onSiteLoaded) onSiteLoaded(sites[current]);
  if (loadWithoutEffect && loadingSiteAnchor != '') scrollTo(loadingSiteAnchor, 2);
 }

 var sitescroll = /* private */ function () { // funktion fuer Scrollevent
  if (!autoChangeAnchor || currentSiteAnchors.length == 0 || scrollInfo['scrolling']) return;
  // richtigen Anker ermitteln
  var pos = window.pageYOffset;
  var nr = anchorPos;
  while (nr > 0 && currentSiteAnchors[nr]['pos'] > pos + 100) nr--;
  while (nr < currentSiteAnchors.length-1 && currentSiteAnchors[nr+1]['pos'] <= pos + 100) nr++;
  // Ankerposition testen
  var top = getAbsOffsetTop(currentSiteAnchors[nr]['el']);
  if (Math.abs(top - currentSiteAnchors[nr]['pos']) > 5) { currentSiteAnchors[nr]['pos'] = top; sitescroll(); return; }
  // Anker wechseln
  if (nr != anchorPos) {
   anchorPos = nr;
   var i = sites[current].location.href.lastIndexOf('#');
   if (i < 0) i = sites[current].location.href.length;
   var stateObj = { foo: "bar" };
   sites[current].history.replaceState(stateObj, '', sites[current].location.href.substring(0,i)+'#'+currentSiteAnchors[nr]['name']);
   communicators[current].newURL(sites[current].location.href.substring(0,i)+'#'+currentSiteAnchors[nr]['name']); // Bugfix: Beim reload der Seite werden im Frame die Funktionen replaceState und pushState nicht ueberwacht
  }
 }

 var autoFrameHeight = /* private */ function (auto) {
  // wenn true uebergeben wird, wird die Hoehe des iframe-Bereiches auf ~100% der Seitenhoehe (das ClassName 'NoJsHeight' wird wieder an alle Elemente, die dies zu Anfang trugen, angehaengt) angepasst, ansonsten richtet sich die Hoehe nach dem Inhalt des aktuellen frames.
  if (auto == isAutoFrameHeight) return;
  isAutoFrameHeight = auto;
  for (var i = 0; i < autoHeightEl.length; i++) {
   var a = autoHeightEl[i].className.split(' ');
   var n = '';
   for (var j = 0; j < a.length; j++) {
    if (a[j] == 'NoJsHeight') continue;
	if (n != '') n += ' ';
	n += a[j];
   }
   autoHeightEl[i].className = n;
  }
  if (auto) {
   for (var i = 0; i < autoHeightEl.length; i++) {
    if (autoHeightEl[i].className != '') autoHeightEl[i].className = autoHeightEl[i].className + ' ';
    autoHeightEl[i].className = autoHeightEl[i].className + 'NoJsHeight';
   }
   frames[current].style.height = '100%';
  }
 }

 var siteResize = /* private */ function (nr, h) {
  if (nr != current || (false && blendInfo['blending'] && !(blendInfo['state'] & 2))) return; // kein Resize fuer noch nicht sichtbare Seiten
  if (blendInfo['blending'] && isAutoFrameHeight && h >= 0) frames[next].style.height = frames[next].offsetHeight + 'px'; // automatisch gegebene Hoehe einfrieren, bevor autoFrameHeight false wird
  autoFrameHeight(h < 0);
 }
 
 var blend = /* private */ function () {
  // Loop fuer Einblendeeffekt
  if (!blendInfo['timer']) blendInfo['timer'] = window.setInterval(blend, 40);
  var t = getTickCount();
  // blendState - Flags:   1 = alter Frame wurde ausgeblendet    2 = neue Seite hat geladen und Verzoegerung ist um     4 = neue Seite wurde eingeblendet
  if ((blendInfo['state'] & 1) != 1 && !blendInfo['browse']) { // Wenn Flag 1 nicht gesetzt (alte Seite ausblenden)
   if (t - blendInfo['start'] < blendOutTime) {
    if (blendOutEffect) blendOutEffect(frames[next], (t - blendInfo['start'])/blendOutTime, false);
   } else {
    blendInfo['state'] += 1;
    frames[next].style.display = 'none';
    frames[next].src = 'about:blank';
    if (blendOutEffect) blendOutEffect(frames[next], -1, false);
   }
  }
  if (((blendInfo['state'] & 2) != 2) && (blendInfo['currentSiteReady'] || t - blendInfo['start'] > maxWaitForSite) && (t - blendInfo['start'] > blendInDelay)) { // Wenn Flag 2 nicht gesetzt und Bedingungen fuer Einblenden von neuer Seite erfuellt (neue Seite anzeigen)
   if (blendInEffect && !blendInfo['browse']) blendInEffect(frames[current], 0, true);
   else if (browseEffect && blendInfo['browse']) browseEffect(frames[current], frames[next], 0, blendInfo['dir']);
   frames[current].style.maxHeight = 'none';
   blendInfo['startDelay'] = t;
   blendInfo['state'] += 2;
   siteResize(current, communicators[current].contentHeight);
   if (blendInfo['browse']) {
    if (blendInfo['dir']) loadingSiteAnchor = communicators[current].contentHeight + frames[current].parentElement.offsetTop - window.innerHeight;
    else loadingSiteAnchor = 0;
   }
   if (blendInfo['browse']) scrollTo(loadingSiteAnchor, 5);
   else if (loadingSiteAnchor != '') scrollTo(loadingSiteAnchor, 1);
  }
  if (((blendInfo['state'] & 4) != 4) && (blendInfo['state'] & 2)) { // wenn Flag 2 gesetzt aber Flag 4 noch nicht (neue Seite einblenden)
   if (blendInfo['browse']) {
    if (t - blendInfo['startDelay'] < browseDuration) {
     if (browseEffect) browseEffect(frames[current], frames[next], (t-blendInfo['startDelay'])/browseDuration, blendInfo['dir']);
    } else {
     blendInfo['state'] |= 5;
     frames[next].style.display = 'none';
     frames[next].src = 'about:blank';
     if (browseEffect) browseEffect(frames[current], frames[next], -1, blendInfo['dir']);
    }
   } else {
    if (t - blendInfo['startDelay'] < blendInTime) {
     if (blendInEffect) blendInEffect(frames[current], (t - blendInfo['startDelay'])/blendInTime, true);
    } else {
     blendInfo['state'] |= 4;
     if (blendInEffect) blendInEffect(frames[current], -1, true);
    }
   }
  }
  if ((blendInfo['state'] & 5) == 5) { // wenn Flag 1 und 4 gesetzt (Blendvorgang beenden)
   if (blendInfo['timer']) window.clearInterval(blendInfo['timer']);
   blendInfo['timer'] = null;
   blendInfo['blending'] = false;
  }
 }
 
 this.loadSite = /* public */ function (src, behavior) { // behavior : 0=normal, 1=kein Effekt, 2=blaettern, 4=nach oben blaettern
  if (!behavior) behavior = 0;
  if (incompatibleBrowser || src == 'about:blank') return;
  src = absolutePath(src);
  if (isNotInWhitelist(src)) return;
  // Wenn neue Seite gleich aktueller Seite, nur scrollTo ausfuehren
  if (getSiteFromURL(src) == currentSite) {
   if (~behavior & 2) scrollTo(getAnchorFromURL(src));
   return;
  }
  // Einstellungen
  loadingSiteAnchor = getAnchorFromURL(src);
  isFirstPage = false;
  loadWithoutEffect = (behavior & 1);
  currentSiteAnchors = new Array();
  anchorPos = 0;
  // Wenn Effekt aktiv, nur neue Seite in aktuellen Frame laden
  if (blendInfo['blending']) { try { sites[current].location.replace(src); } catch (e) { frames[current].src = src; } return; }
  // current und next wechseln
  current = next;
  next = (current == 0)?1:0;
  // neue Seite laden
  if (~behavior & 1) {
   blendInfo['currentSiteReady'] = false;
   blendInfo['blending'] = true;
  }
  try { sites[current].location.replace(src); } catch (e) { frames[current].src = src; }
  // PrintLinks umstellen
  if (current == 0) for (var i = 0; i < printLinks.length; i++) printLinks[i].href = 'javascript:'+aTarget+'.print()';
  else for (var i = 0; i < printLinks.length; i++) printLinks[i].href = 'javascript:'+aTarget2+'.print()';
  // Blenduebergang starten
  if (behavior & 1) {
   frames[next].style.display = 'none';
   frames[current].style.display = 'block';
  } else {
   frames[current].style.maxHeight = '0px';
   frames[current].style.display = 'block';  // display = block ist notqwendig, damit Seite im Frame beginnt zu laden!
   blendInfo['state'] = 0;
   blendInfo['browse'] = (behavior & 2) == 2;
   blendInfo['dir'] = (behavior & 4) == 4;
   blendInfo['start'] = getTickCount();
   blend();
  }
 }
 
 var scroll = /* private */ function () {
  // Loop fuer Scroll-Effekt
  if (!scrollInfo['timer']) scrollInfo['timer'] = window.setInterval(scroll, 40);
  var t = getTickCount();
  var d = scrollInfo['blendEffectTime'];
  if (scrollInfo['start'] + d > t) {
   var s = (t - scrollInfo['start']) / d;
   s = scrollInfo['fkt'](s);
   s = scrollInfo['startPos'] * (1-s) + scrollInfo['endPos'] * s;
   window.scrollTo(window.pageXOffset, s);
  } else {
   scrollInfo['scrolling'] = false;
   if (scrollInfo['timer']) window.clearInterval(scrollInfo['timer'], 40);
   scrollInfo['timer'] = null;
   window.scrollTo(window.pageXOffset, scrollInfo['endPos']);
  }
 }
 
 this.scrollTo = /* public */ function (target, scrollbehavior) { // scrollbehavior: 0=normal, 1=scrollt so lange, wie Einblendeeffekt, 2=springt sofort zu neuer Position (kein Effekt), 4=scrollFkt = browseScrollFkt
  if (!scrollbehavior) scrollbehavior = 0;
  if (incompatibleBrowser) return;
  loadingSiteAnchor = '';
  if (scrollbehavior & 4) scrollInfo['fkt'] = browseScrollFunction;
  else scrollInfo['fkt'] = scrollFunction;
  // Scrollziel suchen
  var pos = 0;
  if (!isNaN(target)) pos = target;
  else {
   try {
    var el = sites[current].document.getElementById(target);
	if (!el) el = sites[current].document.getElementsByName(target)[0];
	if (el) pos = getAbsOffsetTop(el);
   } catch (e) { return; }
   try {
    var L = sites[current].location.href;
    var i = L.lastIndexOf('#');
    if (i > 0 && L.lastIndexOf('?') < i) L = L.substring(0, i);
    var stateObj = { foo: "bar" };
    if (sites[current].location.href != L+'#'+target) sites[current].history.replaceState(stateObj, document.title+': '+sites[current].document.title, L+'#'+target);
   } catch (e) {  }
  }
  // Scrollziel setzen
  scrollInfo['endPos'] = pos;
  if (scrollInfo['scrolling']) return;
  // Scrolleffekt starten
  scrollInfo['scrolling'] = true;
  scrollInfo['start'] = getTickCount();
  if (scrollbehavior & 2) scrollInfo['start'] = 0;
  scrollInfo['startPos'] = window.pageYOffset;
  scrollInfo['blendEffectTime'] = scrollTime;
  if (scrollbehavior & 1) scrollInfo['blendEffectTime'] = blendInTime;
  if (scrollbehavior & 5 == 5) scrollInfo['blendEffectTime'] = browseDuration;
  scroll();
 }
 
 this.enableOverscroll = function () {
  if (overscrollListener) return;
  overscrollListener = createOverscrollListener(document, 1000, function (dir) {
   for (var i = 0; i < menuLinks.length; i++) if (menuLinks[i]['active']) {
    if (dir && i < menuLinks.length - 1) { loadSite(menuLinks[i+1].href, 2); activateLink(menuLinks[i+1]); }
    else if (!dir && i > 0) { loadSite(menuLinks[i-1].href, 6); activateLink(menuLinks[i-1]); }
    return;
   }
  });
 }
 
 create(); // Objekt sollte vollstaendig geladen sein, bevor Create aufgerufen wird
} }