/*

 Explorer von Mitja Stachowiak (mitjastachowiak.de)

 Diese Klasse erzeugt einen Javascript-Dateiexplorer. Sie kuemmert sich allerdings nur um die Darstellung, alle Aktionen (Kopieren, Umbenennen, Loeschen, Hochladen, etc...) muessen ueber
 Ereignisse abgefangen und dann von einer anderen Komponente ausgefuehrt werden. Es ist ratsam, eine Interface-Klasse zu schreiben, die alle Ereignis-Aktionen implementiert und an ein
 serverseitiges Script weiterleitet. Mit der Klasse FileAdminInterface ist solch eine Klasse gegeben, die diesen Explorer an meine FileAdmin-Komponente fuer Perl weiterleitet.

 Jeder vom Explorer angezeigte Eintrag stammt von einem Objekt ab, das alle Elementparameter (Name, Groesse, Erstelldatum, etc..) traegt. Es koennen beliebige Elementparameter
 gewaehlt werden. Welche davon in welcher Reihenfolge und mit welchem Makro angezeigt werden, haengt von den Kategorien ab. Mit addCategory konnen neue Kategorien erzeugt werden.
 Mit setVisible kann festgelegt werden, welche davon in welcher Reihenfolge sichtbar sind.

 Folgende Elementparameter sind vordefiniert und muessen gegeben sein:
  Name : Der Dateiname des Elements WICHTIG: Dateierweiterungen werden automatisch verarbeitet. Um ein Element als Ordner zu klassifizieren, muss der Dateiname mit / aufhoeren. Das zaehlt quasi als Dateierweiterung fuer Verzeichnisse.
  Rights : Die Rechte des Elements. Dies ist eine Zeichenkette, die die folgenden Buchstaben enthalten kann: R,W,H (Read, Write, Hidden). Diese sollten bereits aus Sicht des aktuellen Benutzers gesetzt werden. Hidden bedeutet halbtransparent - Dateien, die fuer diesen Explorer unsichtbar sein sollen, duerfen gar nicht in der Liste enthalten sein.

 Funktionen und eigenschaften:

 function refresh ()
  Laedt den aktuellen Pfad neu.

 function setList (P,L,S)
  Gibt der Klasse eine neue Liste von Elementen, die angezeigt werden sollen
  P: Pfad des geoeffneten Ordners (in der Regel wie Path)
  L: Die Elemente - Array aus Objekten, jedes Objekt traegt als Werte die Elementparameter
  S: Objekt, welches die Elementparameter des geoeffneten Pfades traegt
  
 function addCategory (name,text,fkt,style)
  Erstellt eine neue Kategorie
  name: Name der Kategorie
  text: Text fuer der Kopfzeile der Tabelle
  fkt: (optional) Ein interpretierbares Stueck Javascript-Code, das die Formatierung des Wertes in der Tabelle uebernimmt. Innerhalb dieses Codes ist V der Wert, der in die Tabelle geschrieben werden soll (V kann hier veraendert werden).
  style: (optional) Ein String aus css-Code, das den jewilligen TD-Elementen der Spalte als Style uebergeben wird.

 function setVisible (_categories)
  Legt fest, welche Kategoriern in welcher Reihenfolge sichtbar sein sollen
  _categories: String mit den Namen der Sichtbaren Kategorien (, ist Separator)

 onOpenPath
  Wird aufgerufen, wenn der Explorer die Dateiliste eines Pfades anfordert
  Typ: function(Path); Path: Pfad, der geoeffnet werden soll
  Callback: setList(P,L,S); Beschreibung siehe SetList

 onCreateFolder
  Wird aufgerufen, wenn ein neuer Ordner angelegt werden soll
  Typ: function(Path); Path: Pfad des neu zu erstellenden Ordners
  Callback: es ist ratsam, nach erfolgreichem Erstellen refresh() aufzurufen

 onDelete
  Wird aufgerufen, wenn ein- oder mehrere Elemente geloescht werden sollen
  Typ: function(El); El: Pfade des zu loeschenden Elemente (, ist Separator)
  Callback: es ist ratsam, nach erfolgreichem Loeschen refresh() aufzurufen

 onRename
  Wird aufgerufen, wenn ein Element umbenannt werden soll
  Typ: function(path,newName); path: Pfad des umzubenennenden Elements; newName: neuer Name (Ohne Pfad)
  Callback: es ist ratsam, bei erfolgreicher Umbenennung refresh() aufzurufen

 onMoveCopy
  Wird aufgerufen, wenn Elemente kopiert oder verschoben werden sollen
  Typ: function(copy,newPath,elements); copy: true fuer kopieren, false fuer verschieben; newPath: Zielpfad; elements: Pfade der zu kopierenden/verschiebenden Elemente (, ist Separator)
  Callback: es ist ratsam, bei erfolgreicher Aktion refresh() aufzurufen

 onGetRights
  Wird aufgerufen, wenn der Explorer die detaillierten Rechte von Elementen anfordert. Detailliert bedeutet aus allgemeiner Sicht, also nicht ob dieser Benutzer etwas darf, sondern allgemein welcher Benutzer was darf.
  Typ: function(str); str sind die Elemente, deren Rechte abgefragt werden sollen (, ist Separator)
  Callback: showRightsBox(rightsStr); rightsStr sind die Rechte (& ist Separator fuer Elemente (gleiche Reihenfolge, wie in str); , ist Separator fuer versch. Rechte). Die versch. Rechte in ihrer Reihenfolge sind: Owner (die userID des Besitzers des Elements), Visibility (0=nur FTP, 1=Besitzer/Admin, 2=Jeder halbtransparent, 3=Jeder), Read (0=nur FTP, 1=Besitzer/Admin, 2=Jeder), Write (0=nur FTP, 1=Besitzer/Admin, 2=Jeder), http (Dateisystemberechtigung fuer http-Zugriff - 1=Erlaubt, 2=Verweigert)

 onSetRights
  Wird aufgerufen, wenn die Zugriffsrechte fuer die gewaehlten ELemente veraendert wurden.
  Typ: function(elements,newRights,inherit); elements: Pfade der gewaehlten Elemente (, ist Separator); newRights: String mit neuen Rechten, duch , getrennt, Reihenfolge ist Owner(-2 fuer nicht aendern),Visible,Read,Wirte,httpZugriff; inherit: falls true, sollen fuer das jeweillige Element nur die Rechte gesetzt werden, die sich von denen des Parents unterscheiden.
  Callback: nach erfolgreicher Aenderung sollte refresh() aufgerufen werden.

 onGetUserName
  Wird aufgerufen, wenn der Explorer den Namen zu einer userID anfordert
  Typ: function(uID)
  Callback: _onGetUserName(uName)
  
 onGetUserList
  Wird aufgerufen, wenn der Explorer eine Liste mit allen UserIDs und UserNames abfragen moechte. Wenn onGetUserList null bleibt, kann der Owner eines Elements nicht geaendert werden
  Typ: function(El); El: Pfade der Elemente, deren Owner geaendert werden soll (, ist separator).
  Callback: _onGetUserList(list); list: array aus IDs und Namen (Gerade indizes sind die IDs, die darauffolgenden - die Namen. Rufen Sie _onGetUserList nicht auf, wenn fuer die uebergebenen Elemente der Owner nicht veraendert werden soll!

 onDownloadFiles
  Wird aufgerufen, wenn der Explorer einige Dateien zum Download anbieten moechte.
  Typ: function(El); El: Pfade der herunterzuladenden Dateien (, ist Separator).
  
 onUploadFile
  Wird aufgerufen, wenn der Explorer eine Datei hochladen moechte.
  ?????

 copyIterative
  Boolean - Beeinflusst die Hinweise zum Ersetzen von Dateien. Wenn das serverseitige Script in der Lage ist, Ordner in andere zu integrieren, also nur Elemente zu loeschen, die im neuen Ordner enthalten sind, sollte diese Variable auf true gesetzt werden. Ansonsten wird angezeigt, dass der Server den alten Ordner loeschen wird und dann den neuen kopieren/verschieben wird.

if (!lang) var lang = new Object();
if (!lang['Explorer']) {
 lang['Explorer'] = new Array('de', // first element must specify the Language
  'Neuer Ordner', 'L&ouml;schen', 'Umbenennen', 'Kopieren', 'Ausschneiden', 'Einf&uuml;gen', 'Bereichtigungen', 'Herunterladen', 'Hochladen',
  'M&ouml;chten Sie die ausgew&auml;hlten Elemente wirklich l&ouml;schen?',
  'M&ouml;chten Sie den Ordner "#" in diesen: "#" integrieren (Vorhandene Dateien werden ersetzt)?',
  'M&ouml;chten Sie den Ordner "#" mit diesem: "#" ersetzen (Bestehender Ordner wird vorher gel&ouml;scht)?',
  'M&ouml;chten Sie die Datei "#" mit dieser: "#" ersetzen?',
  'Rechte von', 'Jeder', 'Jeder (halbtransparent)', 'Besitzer\/Admin', 'FTP', 'Ja', 'Nein', 'Nicht ver&auml;nderte Attribute erben',
  'Achtung: Sie sind im Begriff, den Schreibzugriff auf FTP zu setzen. Dies kann nicht &uuml;ber diesen Explorer r&uuml;ckg&auml;ngig gemacht werden!<BR>Sind Sie sicher?',
  'Die angegebene Datei existiert bereits. M&ouml;chten Sie diese mit der hochgeladenen ersetzen?',
  'Abbrechen', 'Server meldet:', 'Ja, alle');
 try { langTranslate('Explorer'); } catch (e) {}
}
function langInsert (str) {
 var a = str.split('#');
 var r = a[0];
 for (var i = 1; i < a.length; i++) {
  if (arguments[i]) r += arguments[i];
  r += a[i];
 }
 return r; 
}

function Explorer (_box) { with (this) {
 // private (Objekte)
 var box = _box;
 var tablebox = document.createElement('div');
 var tableClickBG = document.createElement('div');
 var table = document.createElement('table');
 var bar = document.createElement('div');
 var adressbox = document.createElement('div');
 var adress = document.createElement('input');
 
 // public
 this.rootFolders = new Array();
 
 var _this = this;
 var onEvent = function (el, event, fkt, param, behavior) { // provides easy eventListeners. el: Element to set the listener; event: string-name of the event(without "on"); fkt: a Function to call if the event occours, fkt is called in the scope of _this; param: any parameter, that is given to fkt; behavior: flags - 1=only the param is given to fkt (otherwise there is also given the element and the event), 2=event is not given to the element
  var f;
  if (behavior & 1) f = function (ev) {
    if (!ev) ev = window.event;
 	fkt.call(_this,param);
	if (behavior & 2) { if (ev.preventDefault) ev.preventDefault(); return false; }
   }
  else f = function (ev) {
    if (!ev) ev = window.event;
 	fkt.call(_this,this,ev,param);
    if (behavior & 2) { if (ev.preventDefault) ev.preventDefault(); return false; }
   }
  if (window.addEventListener) el.addEventListener(event, f, false);
  else el.attachEvent('on'+event,f);
  return false;
 }

 var addButton = function (name,text,onclick) {
  var B = document.createElement('a');
  var A;
  A = document.createAttribute('class');
  A.nodeValue = 'Button disabled';
  B.setAttributeNode(A);
  B.href = 'javascript:';
  onEvent(B, 'click', onclick, null, 1);
  B.innerHTML = text;
  bar.appendChild(B);
  but[but.length] = new Object()
  but[but.length-1]['EL'] = B;
  but[but.length-1]['enabled'] = false;
  but[but.length-1]['name'] = name;
  but[but.length-1]['text'] = text;
 }
 var setButtonEnabled = function (name,enabled) {
  for (var i = 0; i < but.length; i++) if (but[i]['name'] == name) {
   var s = 'Button';
   if (enabled == 1) s = s + ' disabled';
   if (enabled == 0) s = s + ' alwaysdisabled';
   but[i]['EL'].className = s;
   but[i]['enabled'] = enabled;
   break;
  }
 }
 var isButtonEnabled = function (name) {
  for (var i = 0; i < but.length; i++) if (but[i]['name'] == name) { return but[i]['enabled'] == 2; }
  return false;
 }

 var create = function () {
  box.className = 'Explorer';
  tablebox.className = 'Box';
  onEvent(tableClickBG, 'mousedown', bgClick);
  tableClickBG.style.position = 'absolute';
  tableClickBG.style.top = '0px';
  tableClickBG.style.left = '0px';
  tableClickBG.style.right = '0px';
  tableClickBG.style.bottom = '0px';
  table.style.position = 'absolute';
  table.style.top = '0px';
  table.style.left = '0px';
  table.style.right = '0px';
  tablebox.appendChild(tableClickBG);
  tablebox.appendChild(table);
  bar.className = 'Bar';
  adressbox.className = 'Adress';
  adress.className = 'Adress';
  onEvent(adress, 'focus', function(){adressFocused=true;});
  onEvent(adress, 'blur', function(){adressFocused=false;});
  adressbox.appendChild(adress);
  backbut.className = 'Back disabled';
  backbut.href = 'javascript:';
  onEvent(backbut, 'click', historyBack);
  forwardbut.className = 'Forward disabled';
  forwardbut.href = 'javascript:';
  onEvent(forwardbut, 'click', historyNext);
  refreshbut.className = 'Refresh';
  onEvent(refreshbut, 'click', refresh);
  upbut.className = 'Up disabled';
  onEvent(upbut, 'click', up);
  hintBox.className = 'Hint';
  hintBox.innerHTML = '<DIV class="bg"></DIV><DIV class="hbox"><DIV class="bar"><A href="javascript:">X</A></DIV><DIV class="text"></DIV></DIV><DIV class="heightSetter">&nbsp;</DIV>';
  hintContent = hintBox.getElementsByTagName('div')[3];
  onEvent(hintBox.getElementsByTagName('a')[0], 'click', closeHint);
  hintBox.style.zIndex = 2;
  addButton('create', lang['Explorer'][1], doCreateFolder);
  addButton('delete', lang['Explorer'][2], doDelete);
  addButton('rename', lang['Explorer'][3], doRename);
  addButton('copy', lang['Explorer'][4], doCopy);
  addButton('cut', lang['Explorer'][5], doCut);
  addButton('paste', lang['Explorer'][6], doPaste);
  addButton('rights', lang['Explorer'][7], doSetRights);
  addButton('download', lang['Explorer'][8], doDownloadFiles);
  addButton('upload', lang['Explorer'][9], doUploadFile);
  box.appendChild(backbut);
  box.appendChild(forwardbut);
  box.appendChild(adressbox);
  box.appendChild(refreshbut);
  box.appendChild(upbut); 
  box.appendChild(bar);
  box.appendChild(tablebox);
  box.appendChild(hintBox);
 }

 this.up = function () {
  var S = history[historyPos];
  if (S.length <= root.length || S.lastIndexOf('\/') == -1) return;
  if (S.lastIndexOf('\/') == S.length - 1 && S.length != 0) S = S.substring(0,S.length-1);
  S = S.substring(0,S.lastIndexOf('\/')+1);
  if (onOpenPath) onOpenPath(S);
 }
 this.refresh = function () {
  var S = adress.value;
  if (S.indexOf(virtualRoot) == 0) S = S.substring(virtualRoot.length,S.length);
  if (S.lastIndexOf('\/') != S.length-1 || S.length == 0) S = S + '\/';
  if (onOpenPath) onOpenPath(S);
  closeHint();
 }
 this.historyBack = function () {
  if (historyPos <= 0) return;
  historyPos--;
  if (onOpenPath) onOpenPath(history[historyPos]);
 }
 this.historyNext = function () {
  if (historyPos >= history.length-1) return;
  historyPos++;
  if (onOpenPath) onOpenPath(history[historyPos]);
 }
 var makeupHistory = function () {
  if (historyPos <= 0) backbut.className = 'Back disabled';
  else backbut.className = 'Back';
  if (historyPos >= history.length-1) forwardbut.className = 'Forward disabled';
  else forwardbut.className = 'Forward';
  if (historyPos >= 0 && historyPos < history.length) adress.value = virtualRoot + history[historyPos];
  else adress.value = '';
  if (root.lastIndexOf('\/') != root.length - 1 || root.length == 0) root = root + '\/';
  if (history[historyPos].length <= root.length) upbut.className = 'Up disabled';
  else upbut.className = 'Up';
 }
 var keydown = function (K) {
  if (_keydown) _keydown(K);
  if (!K) K = window.event;
  if (K.keyCode == 16) multiselect = 1;
  if (K.keyCode == 17) multiselect = 2;
  if (K.keyCode == 13 && adressFocused) refresh();
  if (K.keyCode == 46) doDelete();
 }
 document.onkeydown = keydown;
 var keyup = function (K) {
  if (_keyup) _keyup(K);
  if (!K) K = window.event;
  if (K.keyCode == 16) multiselect = 0;
  if (K.keyCode == 17) multiselect = 0;
  if (K.keyCode == 13) if (activeRow && activeRowFkt) activeRowFkt();
 }
 document.onkeyup = keyup;
 var mouseover = function (El) {
  var C = El.className.split(' ');
  var N = '';
  for (var i = 0; i < C.length; i++) {
   if (C[i] == 'Normal') C[i] = 'Hover';
   if (i > 0 && C[i] != '') N += ' ';
   N += C[i];
  }
  El.className = N;
 }
 var mouseout = function (El) {
  var C = El.className.split(' ');
  var N = '';
  for (var i = 0; i < C.length; i++) {
   if (C[i] == 'Hover') C[i] = 'Normal';
   if (i > 0 && C[i] != '') N += ' ';
   N += C[i];
  }
  El.className = N;
 }
 var thClick = function (El) {
  var up = false;
  if (El.className == 'Active Up') up = true;
  for (var i = 0; i < categories.length; i++) categories[i]['EL'].className = 'Normal';
  if (up) El.className = 'Active Down';
  else El.className = 'Active Up';
  sort();
 }

 var doCreateFolder = function () {
  if (!isButtonEnabled('create')) return;
  if (activeRow && activeRowFkt) if (!activeRowFkt()) return;
  var Row = document.createElement('tr');
  var C,E;
  E = document.createElement('td');
  E.className = 'folder';
  Row.appendChild(E);
  for (var j = 0; j < categories.length; j++) {
   if (!categories[j]['visible']) continue;
   C = categories[j]['name'];
   E = document.createElement('td');
   if (C == 'Name') {
    E.innerHTML = '<INPUT type="text"\/>';
    onEvent(E.getElementsByTagName('input')[0], 'blur', _doCreateFolder);
   }
   Row.appendChild(E);
  }
  table.appendChild(Row);
  if (Row.getElementsByTagName('input').length > 0) Row.getElementsByTagName('input')[0].focus();
  activeRow = Row;
  activeRowFkt = _doCreateFolder;
 }
 var _doCreateFolder = function () {
  if (!activeRow) return true;
  var i = 0;
  while (i < activeRow.getElementsByTagName('td').length) if (activeRow.getElementsByTagName('td')[i].getElementsByTagName('input').length > 0) { i = -i - 1; break; } else i++;
  if (i >= 0) { table.removeChild(activeRow); activeRow = false; return true; }
  i = -i - 1;
  var Name = activeRow.getElementsByTagName('td')[i].getElementsByTagName('input')[0].value;
  if (Name == '' || Name == '\/') { table.removeChild(activeRow); activeRow = false; return true; }
  if (Name.charAt(Name.length-1) == '\/') Name = Name.substr(0,Name.length-1);
  var c;
  var j = 0;
  while (j < Name.length) {
   c = Name.charCodeAt(j);
   if (c < 45 && c != 32) break;
   if (c == 47) break;
   if (c >= 58 && c <= 64) break;
   if (c >= 91 && c <= 96 && c != 95) break;
   if (c >= 123) break;
   j++;
  }
  if (j >= Name.length) for (var k = 0; k < list.length; k++) if ((Name+'\/').toLowerCase() == list[k]['Name'].toLowerCase()) j = 0;
  if (j < Name.length ) {
   activeRow.getElementsByTagName('td')[i].getElementsByTagName('input')[0].style.backgroundColor = '#FFDDDD';
   return false;
  }
  Name = Name + '\/';
  activeRow.getElementsByTagName('td')[i].innerHTML = Name;
  list[list.length] = new Object();
  list[list.length-1]['Name'] = Name;
  list[list.length-1]['EL'] = activeRow;
  if (onCreateFolder) onCreateFolder(history[historyPos] + Name);
  activeRowFkt = false;
  activeRow = false;
  return true;
 }
 
 var doDelete = function () {
  if (!isButtonEnabled('delete')) return;
  var L = '';
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   if (L != '') L += ',';
   L += history[historyPos] + list[i]['Name'];
  }
  showHint(lang['Explorer'][10]+'<BR><A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
  onEvent(hintContent.getElementsByTagName('a')[0], 'click', _doDelete, L, 1);
  onEvent(hintContent.getElementsByTagName('a')[1], 'click', closeHint);
 }
 var _doDelete = function (L) {
  closeHint();
  if (onDelete) onDelete(L);
 }
 
 var doRename = function () {
  if (!isButtonEnabled('rename')) return;
  var i;
  for (i = 0; i < list.length; i++) if (list[i]['active']) break;
  if (i >= list.length || !list[i]['active']) return;
  var el = list[i]['EL'].getElementsByTagName('TD')[1];
  var N = list[i]['Name'];
  while (N.lastIndexOf('/') >= 0) N = N.substring(0,N.length-1);
  el.innerHTML = '<INPUT type="text" value="'+N+'"\/>';
  onEvent(el.getElementsByTagName('input')[0], 'blur', _doRename);
  onEvent(el.getElementsByTagName('input')[0], 'keyup', renameVerify);
  var iel = el.getElementsByTagName('input')[0];
  iel.focus();
  iel.select();
 }
 var renameVerify = function (el) {
  for (var i = 0; i < el.value.length; i++) {
   var c = el.value.charCodeAt(i);
   if ((c == 47 && i != el.value.length-1) || (c != 32 && (c < 45 || c > 57) && (c < 65 || c > 90) && c != 95 && (c < 97 || c > 122))) {
    el.style.backgroundColor = 'FAA';
	return false;
   }
  }
  el.style.backgroundColor = 'FFF';
  return true;
 }
 var _doRename = function (el) {
  var i;
  var P = el.parentElement.parentElement;
  for (i = 0; i < list.length; i++) if (list[i]['EL'] == P) break;
  if (i >= list.length || list[i]['EL'] != P) return;
  if (!renameVerify(el)) { el.parentElement.innerHTML = list[i]['Name']; return; }
  if (onRename) onRename(history[historyPos] + list[i]['Name'], el.value);
  else el.parentElement.innerHTML = el.value;
 }
 
 var doCopy = function () {
  if (!isButtonEnabled('copy')) return;
  copiedElements = new Array(false);
  for (var i = 0; i < list.length; i++) if (list[i]['active']) copiedElements.push(history[historyPos] + list[i]['Name']);
  setButtonEnabled('paste',selfList['Rights'] + 1);
  canPaste();
 }
 
 var doCut = function () {
  if (!isButtonEnabled('cut')) return;
  copiedElements = new Array(true);
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   copiedElements.push(history[historyPos] + list[i]['Name']);
   list[i]['EL'].style.display = 'none';
  }
  canPaste();
 }
 
 var canPaste = function () {
  var E = selfList['Rights'] + 1;
  if (copiedElements.length <= 1) E = 1;
  for (var i = 1; i < copiedElements.length; i++) if (copiedElements[i].length > 0 && copiedElements[i].lastIndexOf('/') == copiedElements[i].length-1 && history[historyPos].indexOf(copiedElements[i]) == 0) {
   E = 1;
   break;
  }
  setButtonEnabled('paste',E);
  return E == 2;
 }
 var doPaste = function (offset) {
  if (!isButtonEnabled('paste')) return;
  if (copiedElements.length <= 1) return;
  if (!offset) offset = 1;
  for (offset = offset; offset < copiedElements.length; offset++) {
   var j = copiedElements[offset].lastIndexOf('/',copiedElements[offset].length-2);
   var s = copiedElements[offset].substring(j+1,copiedElements[offset].length);
   for (var i = 0; i < list.length; i++) {
    if (list[i]['Name'] == s && copiedElements[offset] != history[historyPos] + list[i]['Name']) {
     if (list[i]['Name'].lastIndexOf('/') == list[i]['Name'].length-1) {
      if (copyIterative) showHint(langInsert(lang['Explorer'][11], virtualRoot+copiedElements[offset], virtualRoot+history[historyPos]+list[i]['Name'])+'<BR\/><A href="javascript:">'+lang['Explorer'][26]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
      else showHint(langInsert(lang['Explorer'][12], virtualRoot+history[historyPos]+list[i]['Name'], virtualRoot+copiedElements[offset])+'<BR\/><A href="javascript:">'+lang['Explorer'][26]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
     } else showHint(langInsert(lang['Explorer'][13], virtualRoot+history[historyPos]+list[i]['Name'], virtualRoot+copiedElements[offset])+'<BR\/><A href="javascript:">'+lang['Explorer'][26]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
     onEvent(hintContent.getElementsByTagName('a')[0], 'click', function(v){closeHint(); doPaste(v); }, copiedElements.length, 1);
     onEvent(hintContent.getElementsByTagName('a')[1], 'click', function(v){closeHint(); doPaste(v); }, offset+1, 1);
     onEvent(hintContent.getElementsByTagName('a')[2], 'click', function(v){closeHint(); doPaste(v); }, offset+1, 1);
     return;
    }
   }
  }
  if (onMoveCopy) onMoveCopy(!copiedElements[0],history[historyPos],copiedElements.slice(1));
 }
 
 
 
 var doSetRights = function () {
  if (!isButtonEnabled('rights')) return;
  lastSetRightsEl = new Array();
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   lastSetRightsEl.push(history[historyPos] + list[i]['Name']);
  }
  if (onGetRights) onGetRights(lastSetRightsEl.join(','));
 }
 this.showRightsBox = function (rightsStr) {
  if (!isButtonEnabled('rights')) return;
  if (!rightBox) { // Form erstellen, falls diese noch nicht existiert
   rightBox = new Object();
   rightBox['box'] = document.createElement('div');
   rightBox['box'].className = 'Hint';
   rightBox['box'].innerHTML = '<DIV class="bg"></DIV>\n<DIV class="hbox">\n <DIV class="bar">'+lang['Explorer'][14]+' <SPAN></SPAN><A href="javascript:">X</A></DIV>\n <DIV class="text"></DIV>\n</DIV>\n<DIV class="heightSetter">&nbsp;</DIV>';
   onEvent(rightBox['box'].getElementsByTagName('a')[0], 'click', closeRightsBox);
   rightBox['name'] = rightBox['box'].getElementsByTagName('span')[0];
   rightBox['text'] = rightBox['box'].getElementsByTagName('div')[3];
   rightBox['text'].innerHTML = '\n'+
   ' <FORM name="rightsForm">'+
   '  <TABLE>\n'+
   '   <TR>\n'+
   '    <TD> Sichbarkeit: <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioVisible" value="3">'+lang['Explorer'][15]+' &nbsp;&nbsp;\n'+
   '         <INPUT type="radio" name="radioVisible" value="2">'+lang['Explorer'][16]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioVisible" value="1">'+lang['Explorer'][17]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioVisible" value="0">'+lang['Explorer'][18]+' <\/TD>\n'+
   '   <\/TR>\n'+
   '   <TR>\n'+
   '    <TD> Lesen: <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioRead" value="2">'+lang['Explorer'][15]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioRead" value="1">'+lang['Explorer'][17]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioRead" value="0">'+lang['Explorer'][18]+' <\/TD>\n'+
   '   <\/TR>\n'+
   '   <TR>\n'+
   '    <TD> Schreiben: <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioWrite" value="2">'+lang['Explorer'][15]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioWrite" value="1">'+lang['Explorer'][17]+' <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioWrite" value="0">'+lang['Explorer'][18]+' <\/TD>\n'+
   '   <\/TR>\n'+
   '   <TR>\n'+
   '    <TD> Zugriff &uuml;ber http: <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioHTTP" value="1">'+lang['Explorer'][19]+' <\/TD>\n'+
   '    <TD> <\/TD>\n'+
   '    <TD> <INPUT type="radio" name="radioHTTP" value="0">'+lang['Explorer'][20]+' <\/TD>\n'+
   '   <\/TR>\n'+
   '   <TR>\n'+
   '    <TD> Besitzer: <\/TD>\n'+
   '    <TD> <SPAN><\/SPAN> <SELECT name="owner" size="1" style="display : none; "></SELECT> <\/TD>\n'+
   '    <TD> <\/TD>\n'+
   '    <TD> <\/TD>\n'+
   '   <\/TR>\n'+
   '  <\/TABLE>\n'+
   '  <INPUT type="checkbox" name="checkboxInherit">'+lang['Explorer'][21]+' \n'+
   ' <\/FORM>\n'+
   ' <DIV style="position : absolute; bottom : 0px; left : 0px; right : 0px; background-color : white; border-top : 1px solid grey; text-align : right; padding : 3px; ">\n'+
   '  <INPUT type="button" value="OK">\n'+
   ' <\/DIV>\n';
   onEvent(rightBox['text'].getElementsByTagName('select')[0], 'click', onOwnerChanged);
   onEvent(rightBox['text'].getElementsByTagName('input')[rightBox['text'].getElementsByTagName('input').length-1], 'click', setRightsOK);
   rightBox['form'] = rightBox['text'].getElementsByTagName('form')[0];
   rightBox['owner'] = rightBox['text'].getElementsByTagName('span')[0];
   rightBox['ownerSelect'] = rightBox['text'].getElementsByTagName('select')[0];
   rightBox['ownerTR'] = rightBox['text'].getElementsByTagName('tr')[4];
   box.appendChild(rightBox['box']);
  }
  // Standard-Aussehen setzen
  rightBox['UserList'] = null;
  rightBox['oID'] = -1;
  rightBox['ownerChanged'] = false;
  rightBox['owner'].style.display = 'inline';
  rightBox['ownerSelect'].style.display = 'none';
  rightBox['box'].style.display = 'block';
  for (var i =  0; i < rightBox['form'].getElementsByTagName('input').length; i++) rightBox['form'].getElementsByTagName('input')[i].checked = false;
  rightBox['form'].checkboxInherit.checked = true;
  rightBox['owner'].innerHTML = '';
  if (onGetUserName) rightBox['ownerTR'].style.display = 'table-row';
  else rightBox['ownerTR'].style.display = 'none';
  // Uebergebene Liste mit Rechten interpretieren; Wert < 0 bedeutet: Nicht alle markierten Elemente haben das gleiche Recht
  var O = -1;
  var V = -1;
  var R = -1;
  var W = -1;
  var http = -1;
  var a = rightsStr.split('&');
  for (var i = 0; i < a.length; i++) {
   var r = a[i].split(',');
   if (r[0] == '') O = -2
   else if (O >= 0 && O != r[0]) O = -2
   else if (O == -1) O = r[0];
   if (V >= 0 && V != r[1]) V = -2
   else if (V == -1) V = r[1];
   if (R >= 0 && R != r[2]) R = -2
   else if (R == -1) R = r[2];
   if (W >= 0 && W != r[3]) W = -2
   else if (W == -1) W = r[3];
   if (http >= 0 && http != r[4]) http = -2
   else if (http == -1) http = r[4];
  }
  // Rechte in Form eintragen
  rightBox['name'].innerHTML = lastSetRightsEl.join(', ');
  if (V >= 0) rightBox['form'].getElementsByTagName('input')[3-V].checked = true;
  if (R >= 0) rightBox['form'].getElementsByTagName('input')[6-R].checked = true;
  if (W >= 0) rightBox['form'].getElementsByTagName('input')[9-W].checked = true;
  if (http >= 0) rightBox['form'].getElementsByTagName('input')[11-http].checked = true;
  rightBox['oID'] = O;
  if (O >= 0 && onGetUserName) onGetUserName(O);
  // UserList anfordern
  if (onGetUserList) onGetUserList(lastSetRightsEl.join(','));
 }
 this._onGetUserName = function (uName) {
  if (rightBox && rightBox['owner']) rightBox['owner'].innerHTML = uName;
 }
 this._onGetUserList = function (uList) {
  rightBox['UserList'] = uList;
  var s = '';
  for (var i = 0; i < uList.length - 1; i += 2) {
   s += '<OPTION value="'+uList[i]+'">'+uList[i+1]+'<\/OPTION>';
  }
  rightBox['ownerSelect'].innerHTML = s;
  if (rightBox['oID'] >= 0) { rightBox['ownerSelect'].defaultSelected = true; rightBox['ownerSelect'].value = rightBox['oID']; }
  else { rightBox['ownerSelect'].defaultSelected = false; rightBox['ownerSelect'].selectedIndex = -1; }
  
  rightBox['owner'].style.display = 'none';
  rightBox['ownerSelect'].style.display = 'inline';
 }
 var onOwnerChanged = function () {
  rightBox['ownerChanged'] = true;
 }
 var setRightsOK = function () {
  if (rightBox['form'].getElementsByTagName('input')[9].checked) {
   showHint(lang['Explorer'][22]+'<BR\/><A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
   onEvent(hintContent.getElementsByTagName('a')[0], 'click', function(){_doSetRights(); closeHint(); });
   onEvent(hintContent.getElementsByTagName('a')[1], 'click', closeHint);
  } else _doSetRights();
 }
 var _doSetRights = function () {
  var O = -2; if (rightBox['ownerSelect'] && rightBox['ownerSelect'].value >= 0 && rightBox['ownerChanged']) O = rightBox['ownerSelect'].value;
  var V = 0; while (V < rightBox['form'].radioVisible.length) { if (rightBox['form'].radioVisible[V].checked) break; else V++; } V = rightBox['form'].radioVisible.length - 1 - V;
  var R = 0; while (R < rightBox['form'].radioRead.length) { if (rightBox['form'].radioRead[R].checked) break; else R++; } R = rightBox['form'].radioRead.length - 1 - R;
  var W = 0; while (W < rightBox['form'].radioWrite.length) { if (rightBox['form'].radioWrite[W].checked) break; else W++; } W = rightBox['form'].radioWrite.length - 1 - W;
  var http = 0; while (http < rightBox['form'].radioHTTP.length) { if (rightBox['form'].radioHTTP[http].checked) break; else http++; } http = rightBox['form'].radioHTTP.length - 1 - http;
  var L = '';
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   if (L != '') L += ',';
   L += history[historyPos] + list[i]['Name'];
  }
  if (onSetRights) onSetRights(L,O+','+V+','+R+','+W+','+http,rightBox['form'].checkboxInherit.checked);
  closeRightsBox();
 }
 var closeRightsBox = function () {
  rightBox['box'].style.display = 'none';
 }
 
 var doDownloadFiles = function () {
  if (!isButtonEnabled('download')) return;
  var a = new Array();
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   a.push(history[historyPos] + list[i]['Name']);
  }
  if (onDownloadFiles) onDownloadFiles(a.join(','));
 }
 
 var doUploadFile = function () {
  if (!isButtonEnabled('upload') || !onUploadInit || !onUploadTerminate) return;
  if (activeRow && activeRowFkt) if (!activeRowFkt()) return;
  var Row = document.createElement('tr');
  var C,E;
  E = document.createElement('td');
  E.className = 'dat';
  Row.appendChild(E);
  for (var j = 0; j < categories.length; j++) {
   if (!categories[j]['visible']) continue;
   C = categories[j]['name'];
   E = document.createElement('td');
   E.style.overflow = 'visible';
   if (C == 'Name') {
    E.innerHTML = '<DIV style="width : 0px; height : 0px; white-space : nowrap; "><FORM><INPUT name="file" type="file"\/><INPUT type="button" value="upload"\/><\/FORM></DIV>';
    onEvent(E.getElementsByTagName('input')[1], 'click', _doUploadFile);
   }
   Row.appendChild(E);
  }
  table.appendChild(Row);
  activeRow = Row;
  activeRowFkt = _doUploadFile;
 }
 var _doUploadFile = function () {
  if (!activeRow) return;
  var El = activeRow.getElementsByTagName('input')[0];
  if (!El.files || El.files.length == 0) {
   __doUploadFile(false);
  }
  for (var i = 0; i < list.length; i++) if (list[i]['Name'].toLowerCase() == El.files[0].name.toLowerCase()) {
   showHint(lang['Explorer'][23]+'<BR\/><A href="javascript:">'+lang['Explorer'][19]+'<\/A>&nbsp;&nbsp;<A href="javascript:">'+lang['Explorer'][20]+'<\/A>');
   onEvent(hintContent.getElementsByTagName('a')[0], 'click', function(){__doUploadFile(true); closeHint(); });
   onEvent(hintContent.getElementsByTagName('a')[1], 'click', function(){__doUploadFile(false); closeHint(); });   
   return;
  }
  __doUploadFile(true);
 }
 var __doUploadFile = function (upload) {
  var El = activeRow.getElementsByTagName('input')[0];
  if (!upload) {
   table.removeChild(activeRow);
   activeRow = null;
   return;
  }
  var div = activeRow.getElementsByTagName('div')[0];
  div.innerHTML = '<FORM><DIV style="display : inline-block; vertical-align : middle; ">'+El.files[0].name+'<\/DIV> <DIV style="height : 1em; width : 200px; border : 1px solid grey; display : inline-block; vertical-align : middle; overflow : hidden; "><DIV style="width : 0px; height : 100%; background-color : blue; "><\/DIV><\/DIV> <INPUT type="button" value="'+lang['Explorer'][24]+'" style="vertical-align : top; "\/><\/FORM>';
  var nr = uploadOpenFile(El,div);
  onEvent(div.getElementsByTagName('input')[0], 'click', upload_Aboard, nr, 1);
  activeRow = null;
 }

 // Upload-Loop functions
 var encodeCharset = new Array('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z','_','-');
 var sizePerUpload = 4095; // Anzahl an Soll-Bytes pro Request - sollte ein Vielfaches von 3 sein, da immer 3 Bytes zu 4 Zeichen gepackt werden.
 var uploadInf = new Array();
 var uploadOpenConnection = function (nr) { // Hierein der Code fuer Aktionen die fuer den Aufbau der Verbindung noetig sind
  uploadInf[nr]['fullFileName'] = history[historyPos]+uploadInf[nr]['fileName'];
  onUploadInit(nr,uploadInf[nr]['fullFileName']);
 }
 var uploadPercent = function (nr,p) { // Hierein der Code fuer Aktionen bei Upload-Fortschritt
  uploadInf[nr]['param'].getElementsByTagName('div')[2].style.width = (p*100)+'%';
  if (p >= 0.1 && !uploadInf[nr]['tableReloaded']) {
   uploadInf[nr]['tableReloaded'] = true;
   refresh();
  }
 }
 var uploadError = function (nr,errorMsg) { // Hierein der Code fuer Aktionen nach Error
  trEl = uploadInf[nr]['param'].parentElement.parentElement;
  for (var i = 0; i < list.length; i++) if (list[i]['EL'] == trEl) { list.splice(i,1); break; }
  table.removeChild(trEl);
  uploadInf[nr] = null;
  showHint('Fehler beim Upload: '+errorMsg);
 }
 var uploadSuccess = function (nr) { // Hierein der Code fuer Aktionen nach erfolgreichem Upload
  var tdEl = uploadInf[nr]['param'].parentElement;
  tdEl.removeChild(uploadInf[nr]['param']);
  tdEl.innerHTML = uploadInf[nr]['fileName'];
  var trEl = tdEl.parentElement;
  for (var i = 0; i < trEl.getElementsByTagName('td').length; i++) trEl.getElementsByTagName('td')[i].style.visibility = 'visible';
  for (var i = 0; i < list.length; i++) if (list[i]['EL'] == trEl) { list[i]['Rights'] = 'RW'; break; }
  uploadInf[nr] = null;
 }
 var uploadAboard = function (nr) { // Hierein der Code fuer Aktionen nach Abbruch
  trEl = uploadInf[nr]['param'].parentElement.parentElement;
  for (var i = 0; i < list.length; i++) if (list[i]['EL'] == trEl) { list.splice(i,1); break; }
  table.removeChild(trEl);
  uploadInf[nr] = null;
 }
 var upload_Aboard = function (nr) { // Aufrufen, um Upload abzubrechen
  uploadInf[nr]['success'] = 1;
 }
 this._uploadInit = function (nr,key,scriptPath) { // Als Response von onUploadInit aufrufen
  uploadInf[nr]['scriptPath'] = scriptPath;
  uploadInf[nr]['scriptKey'] = key;
  uploadPercent(nr,0.1);
  uploadNext(nr);
 }
 this._uploadTerminate = function (nr) { // Als Response von onUploadTerminate aufrufen
  if (!uploadInf[nr]) return;
  if (uploadInf[nr]['success'] == 2) { uploadPercent(nr,1); uploadSuccess(nr); }
  else if (uploadInf[nr]['success'] == 1) uploadAboard(nr);
 }
 var uploadTerminate = function (nr,success) {
  onUploadTerminate(nr,uploadInf[nr]['scriptKey'],uploadInf[nr]['scriptPath'],success);
 }
 var upload_Error = function (nr,errorMsg) { // wenn waehrend des Uploads (ausser bei terminate) ein unerwarteter Fehler auftritt
  uploadInf[nr]['success'] = 0;
  uploadTerminate(nr,false);
  uploadError(nr,errorMsg);
 }
 var uploadNext = function (nr) {
  var size = sizePerUpload;
  if (uploadInf[nr]['pos'] + size > uploadInf[nr]['size']) size = uploadInf[nr]['size'] - uploadInf[nr]['pos'];
  if (size <= 0) return;
  var buf = new Uint8Array(uploadInf[nr]['buffer'], uploadInf[nr]['pos'], size);
  uploadInf[nr]['pos'] += size;
  var p = 0;
  var c;
  var s = '?'+uploadInf[nr]['scriptKey']+'&'+size+'&';
  while (p < size) {
   c = buf[p]; p++;
   if (p < size) c += buf[p]*256; p++;
   if (p < size) c += buf[p]*65536; p++;
   s += encodeCharset[c % 64]; c = c/64 >> 0;
   s += encodeCharset[c % 64]; c = c/64 >> 0;
   s += encodeCharset[c % 64]; c = c/64 >> 0;
   s += encodeCharset[c];
  }
  uploadInf[nr]['xmlhttp'].open('post', uploadInf[nr]['scriptPath'] + s, true);
  uploadInf[nr]['xmlhttp'].send();
 }
 var uploadRequestResponse = function (nr) {
  if (!uploadInf[nr]) return;
  if (uploadInf[nr]['xmlhttp'].readyState != 4) { return; }
  if (uploadInf[nr]['xmlhttp'].status != 200) { upload_Error(nr,'Server error code: ' + uploadInf[nr]['xmlhttp'].status); return; }
  if (uploadInf[nr]['xmlhttp'].responseText.indexOf('success') < 0) { upload_Error(nr,lang['Explorer'][25] + uploadInf[nr]['xmlhttp'].responseText); return; }
  uploadPercent(nr,(uploadInf[nr]['pos'] / uploadInf[nr]['size']) * 0.85 + 0.1);
  if (uploadInf[nr]['success'] && uploadInf[nr]['success'] == 1) { uploadTerminate(nr,false); return; }
  if (uploadInf[nr]['pos'] < uploadInf[nr]['size']) uploadNext(nr);
  else { uploadPercent(nr,0.97); uploadInf[nr]['success'] = 2; uploadTerminate(nr,true); }
 }
 var uploadOpenFile = function (El,param) {
  var nr = 0;
  while (nr < uploadInf.length) if (uploadInf[nr] == null) break; else nr++;
  if (!window.FileReader() || !window.XMLHttpRequest) { upload_Error(nr,'Browser not supported'); return; }
  uploadInf[nr] = new Object();
  uploadInf[nr]['param'] = param;
  uploadPercent(nr,0);
  uploadInf[nr]['fileName'] = El.files[0].name;
  uploadInf[nr]['file'] = El.files[0];
  uploadInf[nr]['reader'] = new FileReader();
  if (window.XMLHttpRequest) uploadInf[nr]['xmlhttp'] = new XMLHttpRequest();
  else uploadInf[nr]['xmlhttp'] = new ActiveXObject("Microsoft.XMLHTTP");
  uploadInf[nr]['xmlhttp'].onreadystatechange = function () { uploadRequestResponse(nr); }
  function readerLoad (Ev) {
   uploadInf[nr]['buffer'] = uploadInf[nr]['reader'].result;
   uploadInf[nr]['size'] = uploadInf[nr]['buffer'].byteLength;
   uploadInf[nr]['pos'] = 0;
   uploadPercent(nr,0.05);
   uploadOpenConnection(nr);
  }
  uploadInf[nr]['reader'].onload = readerLoad;
  uploadInf[nr]['reader'].onerror = function () { upload_Error(nr,'FileReader Error, code: '+uploadInf[nr]['reader'].error); }
  uploadInf[nr]['reader'].readAsArrayBuffer(uploadInf[nr]['file']);
  return nr;
 }

 this.sort = function (param) {
  var i;
  if (param) {
   for (i = 0; i < categories.length; i++) if (categories[i]['name'] == param) thClick(categories[i]['EL']);
  }
  i = 0;
  while (i < categories.length) {
   if (categories[i]['EL'] && categories[i]['EL'].className.indexOf('Active') >= 0) break;
   else i++;
  }
  if (i == categories.length && lastActiveCat != 0 && Math.abs(lastActiveCat)-1 < categories.length && categories[Math.abs(lastActiveCat)-1]['visible']) {
   i = Math.abs(lastActiveCat)-1;
   if (lastActiveCat > 0) categories[i]['EL'].className = 'Active Up';
   else categories[i]['EL'].className = 'Active Down';
  }
  if (i == categories.length) return;
  lastActiveCat = i + 1;
  var F = false;
  var N = categories[i]['name'];
  var dir = false;
  if (categories[i]['EL'].className.indexOf('Up') >= 0) dir = true;
  else lastActiveCat = -lastActiveCat;
  if (N == 'Name') F = true;
  var Err = false;
  function AgtB (A,B) { try {
   if (!A) A = 0;
   if (!B) B = 0;
   A = A.toLowerCase();
   B = B.toLowerCase();
   if (F) {
    var C = getExtension(A) == 'folder';
    var D = getExtension(B) == 'folder';
    if (D && !C) return true;
    if (!D && C) return false;
   }
   if (A.charCodeAt(0) >= 48 && A.charCodeAt(0) < 58 && B.charCodeAt(0) >= 48 && B.charCodeAt(0) < 58) {
    A = parseInt(A);
    B = parseInt(B);
   }
   return A > B;
  } catch (e) { Err = true; return false; } }
  var s = false;
  var n = 0;
  var E,C1,C2,k;
  while (!s) {
   s = true;
   for (var j = 1; j < table.getElementsByTagName('tr').length - 1; j++) {
    E = table.getElementsByTagName('tr')[j];
    k = 0;
    while (k < list.length) {
     if (list[k]['EL'] == E) { C1 = list[k][N]; break; }
     else k++;
    }
    E = table.getElementsByTagName('tr')[j+1];
    k = 0;
    while (k < list.length) {
     if (list[k]['EL'] == E) { C2 = list[k][N]; break; }
     else k++;
    }
    if (AgtB(C1,C2) == dir) {
     if (Err) return;
     table.removeChild(E);
     table.insertBefore(E,table.getElementsByTagName('tr')[j]);
     s = false;
    }
   }
   n++;
   if (n > list.length + 10) break;
  }
 }

 var getExtension = function (N) {
  if (N.lastIndexOf('\/') == N.length - 1 && N.length != 0) return 'folder';
  if (N.lastIndexOf('.jpg') == N.length-4 || N.lastIndexOf('.gif') == N.length-4 || N.lastIndexOf('.png') == N.length-4) return 'img';
  if (N.lastIndexOf('.wav') == N.length-4 || N.lastIndexOf('.ogg') == N.length-4 || N.lastIndexOf('.mp3') == N.length-4) return 'audio';
  if (N.lastIndexOf('.html') == N.length-5 || N.lastIndexOf('.txt') == N.length-4 || N.lastIndexOf('.pdf') == N.length-4) return 'text';
  return 'dat';
 }

 var show = function () {
  table.innerHTML = '';
  activeRow = false;
  activeRowFkt = false;
  // Tabelle erstellen
  var A,E,C;
  E = document.createElement('tr');
  C = document.createElement('th');
  A = document.createAttribute('class');
  A.nodeValue = 'dat';
  C.setAttributeNode(A);
  E.appendChild(C);
  for (var i = 0; i < categories.length; i++) {
   if (!categories[i]['visible']) continue;
   A = document.createAttribute('class');
   A.nodeValue = 'Normal';
   categories[i]['EL'].setAttributeNode(A);
   onEvent(categories[i]['EL'], 'mouseover', mouseover);
   onEvent(categories[i]['EL'], 'mouseout', mouseout);
   onEvent(categories[i]['EL'], 'mousedown', thClick);
   categories[i]['EL'].innerHTML = categories[i]['text'];
   E.appendChild(categories[i]['EL']);
  }
  table.appendChild(E);
  for (var i = 0; i < list.length; i++) {
   list[i]['EL'] = document.createElement('tr');
   A = document.createAttribute('class');
   A.nodeValue = 'Normal';
   if (list[i]['Rights'] && list[i]['Rights'].indexOf('H') >= 0) A.nodeValue += ' Hidden';
   list[i]['EL'].setAttributeNode(A);
   onEvent(list[i]['EL'], 'mouseover', mouseover);
   onEvent(list[i]['EL'], 'mouseout', mouseout);
   E = document.createElement('td');
   A = document.createAttribute('class');
   A.nodeValue = 'dat';
   E.setAttributeNode(A);
   list[i]['EL'].appendChild(E);
   for (var j = 0; j < categories.length; j++) {
    if (!categories[j]['visible']) continue;
    C = categories[j]['name'];
    if (C == 'Name') list[i]['EL'].getElementsByTagName('td')[0].className = getExtension(list[i][C]);
    E = document.createElement('td');
    function makeup (V,Ev) {
     try { eval(Ev); }
     catch (e) { V = 'ERROR'; }
     return V;
    }
    E.innerHTML = makeup(list[i][C],categories[j]['fkt']);
    if (categories[j]['style']) {
     A = document.createAttribute('style');
     A.nodeValue = categories[j]['style'];
     E.setAttributeNode(A);
    }
	onEvent(E, 'mousedown', tdClick, i, 1);
	onEvent(E, 'dblclick', tdDblClick, i, 1);
    list[i]['EL'].appendChild(E);
   }
   for (var j = 0; j < uploadInf.length; j++) if (uploadInf[j] && uploadInf[j]['fullFileName'].toLowerCase() == (history[historyPos]+list[i]['Name']).toLowerCase()) {
    var m = 0;
    for (var k = 0; k < categories.length; k++) if (categories[k]['visible']) {
	 m++;
	 if (categories[k]['name'] == 'Name') { list[i]['EL'].getElementsByTagName('td')[m].innerHTML = ''; list[i]['EL'].getElementsByTagName('td')[m].appendChild(uploadInf[j]['param']); }
	 else list[i]['EL'].getElementsByTagName('td')[m].style.visibility = 'hidden';
	}
   }
   table.appendChild(list[i]['EL']);
  }
  sort();
 }

 var tdDblClick = function (Row) {
  if (getExtension(list[Row]['Name']) == 'folder') {
   var S = history[historyPos];
   if (S.lastIndexOf('\/') != S.length - 1 || S.length == 0) S = S + '\/';
   S = S + list[Row]['Name'];
   if (onOpenPath) onOpenPath(S);
  }
 }
 
 var setOrRemoveClassAtr = function (obj,atr,set) {
  var C = obj.className.split(' ');
  var N = '';
  for (var i = 0; i < C.length; i++) {
   if (C[i] == atr) C[i] = '';
   if (i > 0 && C[i] != '') N += ' ';
   N += C[i];
  }
  if (set) N += ' ' + atr;
  obj.className = N;
 }

 var tdClick = function (Row) { // Row ist die nummer des Listen-Elements, das angeklickt wurde, aber nicht die ummer des TR-Elements.
  function getListIdxByEl (El) {
   for (var i = 0; i < list.length; i++) if (list[i]['EL'] == El) return i;
   return -1;
  }
  if (multiselect == 2) { // Einzele selektieren und deselektieren
   if (!list[Row]['active']) { list[Row]['active'] = true; setOrRemoveClassAtr(list[Row]['EL'],'Active',true); }
   else { list[Row]['active'] = false; setOrRemoveClassAtr(list[Row]['EL'],'Active',false); }
  } else if (multiselect == 1) { // Von bis selektieren
   var i = 1;
   var tr = table.getElementsByTagName('tr');
   while (i < tr.length) { if (list[getListIdxByEl(tr[i])]['active']) break; i++; }
   var RowIdx = 0;
   while (RowIdx < tr.length && list[Row]['EL'] != tr[RowIdx]) RowIdx++;
   while (i != RowIdx) { list[getListIdxByEl(tr[i])]['active'] = true; setOrRemoveClassAtr(tr[i],'Active',true); if (RowIdx < i) i--; if (RowIdx > i) i++; }
   setOrRemoveClassAtr(list[Row]['EL'],'Active',true);
   list[Row]['active'] = true;
  } else { // Nur ein Element selektieren
   for (var i = 0; i < list.length; i++) if (list[i]['active']) { list[i]['active'] = false; setOrRemoveClassAtr(list[i]['EL'],'Active',false); }
   setOrRemoveClassAtr(list[Row]['EL'],'Active',true);
   list[Row]['active'] = true;
  }
  checkActionsForActiveElements();
 }
 var bgClick = function () {
  for (var i = 0; i < list.length; i++) if (list[i]['active']) { list[i]['active'] = false; setOrRemoveClassAtr(list[i]['EL'],'Active',false); }
  checkActionsForActiveElements();
 }


 this.showHint = function (text) {
  hintContent.innerHTML = text;
  hintBox.style.display = 'block';
 }
 this.closeHint = function () {
  hintBox.style.display = 'none';
 }
 
 var checkActionsForActiveElements = function () {
  var filesOnly = true;
  var parentWrite = true;
  var read = true;
  var write = true;
  var n = 0;
  for (var i = 0; i < list.length; i++) if (list[i]['active']) {
   n++;
   if (list[i]['Name'].indexOf('/') >= 0) filesOnly = false;
   if (!list[i]['Rights']) continue;
   if (list[i]['Rights'].indexOf('R') < 0) read = false;
   if (list[i]['Rights'].indexOf('W') < 0) write = false;
  }
  if (n == 0) { read = false; write = false; }
  if (selfList['Rights'] == 0) parentWrite = false;
  if (write && parentWrite) setButtonEnabled('delete',2); else setButtonEnabled('delete',1);
  if (write && parentWrite && n == 1) setButtonEnabled('rename',2); else setButtonEnabled('rename',1);
  if (read && write && parentWrite) setButtonEnabled('cut',2); else setButtonEnabled('cut',1);
  if (read) setButtonEnabled('copy',2); else setButtonEnabled('copy',1);
  if (write) setButtonEnabled('rights',2); else setButtonEnabled('rights',1);
  if (read && filesOnly) setButtonEnabled('download',2); else setButtonEnabled('download',1);
  if (parentWrite) setButtonEnabled('upload',2); else setButtonEnabled('upload',1);
 }

 this.setList = function (P,L,S) {
  list = L;
  selfList = S;
  checkActionsForActiveElements();
  if (P.indexOf('\/') == 0) P = P.substring(1,P.length);
  if (history.length == 0 || history[historyPos] != P) {
   closeHint();
   historyPos++;
   history[historyPos] = P;
   while (history.length > historyPos + 1) history.pop();
  }
  makeupHistory();
  setButtonEnabled('create',selfList['Rights'] + 1);
  canPaste();
  show();
 }

 this.addCategory = function (name,text,fkt,style) {
  var i = categories.length;
  categories[i] = new Object();
  categories[i]['name'] = name;
  categories[i]['text'] = text;
  if (fkt) categories[i]['fkt'] = fkt;
  else categories[i]['fkt'] = '';
  if (style) categories[i]['style'] = style;
  categories[i]['visible'] = false;
  categories[i]['EL'] = document.createElement('th');
 }

 this.setVisible = function (_categories) {
  var C = _categories.split(',');
  for (var i = 0; i < categories.length; i++) {
   categories[i]['visible'] = false;
   for (var j = 0; j < C.length; j++) if (C[j] == categories[i]['name']) { categories[i]['visible'] = true; break; }
  }
  show();
 }
 
 create();

} } */

new (function () {
 var unitObj = jsw.units['explorer'];
 // type



function Environment () { with (this) {
 // public
 this.rootInterfaces = new Array();
} }





function Component (environment, parent) {
 // initialization
 this.sync[0] = this;
 this.environment = environment;
 this.el = new Object();
 this.createEl('Box', 'div', parent);
 jsw.units.switchableDesign.unit.requireDesignFile('explorer', this.el.Box);
}
Component.prototype = {
 // protected
 environment : null,
 el : null,
 sync : [null],
 // public
 _currentFolder : null, get currentFolder () { return this._currentFolder; }, set currentFolder (v) { this.open(v); },

 createEl : /* protected */ function (name, tag, parent) {
  this.el[name] = document.createElement(tag);
  this.el[name].className = 'Explorer ' + name;
  if (parent) parent.appendChild(this.el[name]);
 },
 show : /* protected */ function (folder) {
  
 },
 open : /* public */ function (folder) {
  if (!folder.childElements) throw 'DetailedList.open can only open folders!';
  if (this.environment.rootInterfaces.indexOf(folder.rootInterface) == -1) throw "The folder to be opened is not a subelement of this component's environment!";
  for (var i = 0; i < this.sync.length; i++) this.sync[i].show(folder);
 }
}
ComponentPrototype = function () { }
ComponentPrototype.prototype = Component.prototype;




function AdressManager (environment, parent) {
 // initialization
 Component.call(this, environment, parent); // inherited
 this.el.Box.className += ' AdressManager';
 this.createEl('InnerBox', 'div', this.el.Box);
 this.createEl('Edit', 'input', this.el.InnerBox);
 this.el.Edit.type = 'text';
}
AdressManager.prototype = new ComponentPrototype();
AdressManager.prototype._super = Component.prototype;





function DetailedList (environment, parent) {
 // initialization
 Component.call(this, environment, parent); // inherited
 this.el.Box.className += ' DetailedList';
 //createEl('TableBox', 'div', this.el.Box);
 //createEl('Table', 'table', this.el.TableBox);
 this.el.Box.innerHTML = 'DetailedList';
}
DetailedList.prototype = new ComponentPrototype();
DetailedList.prototype._super = Component.prototype;





function TiledList (environment, parent) {
 // initialization
 Component.call(this, environment, parent); // inherited
 this.el.Box.className += ' TiledList';
 this.el.Box.innerHTML = 'Kacheln';
}
TiledList.prototype = new ComponentPrototype();
TiledList.prototype._super = Component.prototype;





function TreeView (environment, parent) {
 // initialization
 Component.call(this, environment, parent); // inherited
 this.el.Box.className += ' TreeView';
 this.el.Box.innerHTML = 'TreeView';
}
TreeView.prototype = new ComponentPrototype();
TreeView.prototype._super = Component.prototype;





function Element (rootInterface, parent, name) {
 // public readonly
 this.rootInterface = rootInterface;
 this.parent = parent;
 this.name = name;
}
Element.prototype = {
 // private
 // public
 get path () {
  if (this.parent = null) return this.rootInterface.name + ':';
  return parent.path + '/' + name;
 },
 disconnect : function () {
  this.parent = null;
  return true;
 }
}
ElementPrototype = function () { }
ElementPrototype.prototype = Element.prototype;





function File (rootInterface, parent, name) {
 Element.call(this, rootInterface, parent, name); // inherited
}
File.prototype = new ElementPrototype();
File.prototype._super = Element.prototype;





function Folder (rootInterface, parent, name) {
 Element.call(this, rootInterface, parent, name); // inherited
}
Folder.prototype = new ElementPrototype();
Folder.prototype._super = Element.prototype;
Folder.prototype.childElements = new Array(); // ist zunaechst leer, wird durch refresh aktualisiert
Folder.prototype.disconnect = function () {
 for (var i = 0; i < this.childElements.length; i++) if (!childElements[i].disconnect()) return false;
 this.childElements = new Array();
 return this._super.disconnect.call(this); // inherited
}





function RootInterface (name) {
 this.name = name;
 this.rootElement = new Folder(this, null, '');
}
RootInterface.name = 'UNDEFINED STORAGE';
RootInterface.prototype.actualizeFolder = function (path, elements) {
};




 // public
 this.Environment = Environment;
 this.Component = Component;
 this.DetailedList = DetailedList;
 this.TiledList = TiledList;
 this.TreeView = TreeView;
 this.Element = Element;
 this.File = File;
 this.Folder = Folder;
 this.RootInterface = RootInterface;
 this.AdressManager = AdressManager;
 this.synchronizeComponents = function (/* args */) {
  for (var i = 0; i < arguments.length; i++) {
   if (!arguments[i].sync) throw 'Only Explorer-Components can be synchronized';
   if (arguments[i].sync.length > 1) throw 'Components can only be synchronized once';
   arguments[i].sync = arguments[i];
  }
 }
 // initialization
 this.free = function () {
  for (var key in this) if (typeof this[key] == 'object') this[key] = null;
 }
 // unit loaded
 unitObj.unit = this;
 unitObj.onload();
})();